﻿using System;
using System.Collections.Generic;
using System.Text;

namespace _04_1_VerketteteStrukturen2
{
    class StringListe
    {
        #region Hilfsklasse für das einzelne Listenelement
        // Die Klasse Element ist  eine *private* Klasse der Klasse StringListe
        // Die Methoden von StringListe dürfen auf private Member zugreifen
        // Methoden außerhalb von StringListe dürfen dies nicht!
        private class Element  // Element ist eine private Hilfsklasse der Klasse StringListe
        {
            public string name;
            public Element next = null;
            public Element(string name) { this.name = name; }
            public override string ToString() => name;
        }
        #endregion

        Element anf = null, ende = null;
        int anz = 0;      // Sehr sinnvoll für Index-Ops

        #region Elemente aufzählen
        public IEnumerator<string> GetEnumerator()
        {
            for (Element item = anf; item != null; item = item.next)
            {
                yield return item.name; // Merken dieser Ausführungsposition
                                        // UND Zurückliefern von item.name
                                        // Beim nächsten Aufruf von GetEnumerator() wird
                                        // an der gespeicherten Pos. weitergemacht.
            }
        }
        // GetEnumerator() --> IEnumerator
        // alle anderen    --> IEnumerable
        public IEnumerable<string> Filter(string text)
        {
            for (Element item = anf; item != null; item = item.next)
            {
                if (item.name.Contains(text))
                    yield return item.name; 
            }
        }
        #endregion

        #region Elemente hinzufügen
        public void AddEnd(string name)
        {
            Element neuesElement = new Element(name);   // 1. Neues Element anlegen
            anz++;
            if (anf == null)                            // 2. Leere Liste?
                anf = ende = neuesElement;
            else
            {
                ende.next = neuesElement;               // 3. Neues Element am Ende anfügen
                ende = ende.next;
            }
        }
        public void AddFront(string name)
        {
            // 1. Leere Liste
            // 2. Es existiert mind. ein Element
            Element neuesElement = new Element(name);   // Neues Element anlegen
            anz++;
            if (anf == null)                            // 1.Fall: Leere Liste?
                anf = ende = neuesElement;
            else
            {
                neuesElement.next = anf;                // 2. Fall
                anf = neuesElement;
            }
        }
        public void AddBeforeNth(int index, string name)
        {
            if (index < 0 || index > anz)
                throw new ArgumentOutOfRangeException("Index außerhalb der Liste");
            if (index == 0)
                AddFront(name);
            else if (index == anz)
                AddEnd(name);
            else
            {
                Element neu = new Element(name);
                anz++;
                Element item = anf;
                for (int i = 1; i < index; i++)
                    item = item.next;
                neu.next = item.next;    // Die Reihenfolge dieses und des nächsten Befehls ist ENTSCHEIDEND!
                item.next = neu;
            }
        }
        public void AddSorted(string name)
        {
            // Fall 1: Leere Liste
            // Fall 2: Anfügen am Listenende
            // Fall 3: Einfügen am Listenanfang
            // Fall 4: Einfügen mittendrin

            // a.CompareTo(b) => neg. gdw. a <  b       eigentlich a-b
            //                => 0    gdw. a == b
            //                => pos. gdw. a >  b

            if (anf == null || ende.name.CompareTo(name) <= 0)  // ende.name < name?
                AddEnd(name);
            else if (name.CompareTo(anf.name) <= 0)
                AddFront(name);
            else
            {
                // Hier wissen wir:
                // Das neue Element ist nicht das erste und nicht das letzte Element
                Element neu = new Element(name);
                anz++;

                Element item = anf;
                while (item.next.name.CompareTo(name) < 0)
                    item = item.next;
                // Findet dasjenige Element, dessen Nachfolgername erstmalig größer
                // als der Name des neuen Elements ist; Hier: item = "A", item.next = "C"
                neu.next = item.next; // "C"
                item.next = neu;
            }
        }
        #endregion

        #region Elemente löschen
        public void DeleteFirst()
        {
            // 1.Fall: Liste ist leer
            // 2.Fall: Liste besteht nur aus einem Element
            // 3.Fall: Liste hat mehr als ein Element
            if (anz == 0)
                return;

            anz--;
            if (anz == 0)
                anf = ende = null;
            else // Wir wissen: Die Liste enthält MEHR als ein Element!
                anf = anf.next;
        }
        public void DeleteLast()
        {
            // 1.Fall: Liste ist leer
            // 2.Fall: Liste besteht nur aus einem Element
            // 3.Fall: Liste hat mehr als ein Element
            if (anz == 0)       // Fall 1
                return;

            anz--;
            if (anz == 0)       // Fall 2
                anf = ende = null;
            else // Wir wissen: Die Liste enthält MEHR als ein Element!
            // d.h. es gibt ein vorletztes Element mit vorletzter.next == ende
            {
                Element vorletzter = anf;
                while (vorletzter.next != ende)
                    vorletzter = vorletzter.next;

                vorletzter.next = null;
                ende = vorletzter;
            }
        }
        public void DeleteNth(int index)
        {
            // 1.Fall: Liste ist leer ODER Ind>=anz --> Fehler
            // 2.Fall: Ind == 0  --> DeleteFirst
            // 3.Fall: Ind == anz-1 --> DeleteLast
            // 4. Bis hierher: Liste hat mind. ein Element und das zu löschende Element
            //                 ist nicht das Erste oder Letzte
            if (anz == 0 || index >= anz)  // 1. Fall
                throw new ArgumentOutOfRangeException("Außerhalb der Anzahl der Listenelemente");
            if (index == 0)  // 2. Fall
                DeleteFirst();
            else if (index == anz - 1) // 3. Fall
                DeleteLast();
            else
            {
                Element vorletzter = anf;
                for (int i = 0; i < index - 1; i++)  // Liste bis zum n-1-ten Element durchlaufen
                    vorletzter = vorletzter.next;
                vorletzter.next = vorletzter.next.next;
                anz--;
            }
        }
        public void DeleteByName(string DelName)
        {
            // Fall 1: leere Liste
            // Fall 2: Erstes Element soll gelöscht werden
            // Fall 3: Ein Element außer dem ersten soll gelöscht werden
            // Fall 4: Das zu löschende Element ist das letzte Element

            if (anf == null)  // Fall 1
                return;
            if (anf.name.CompareTo(DelName) == 0)  // Fall 2
                DeleteFirst();
            else
            {
                // Wir wissen jetzt:
                // Liste hat mindestens ein Element UND das erste Element ist nicht das gesuchte Element!!!
                Element vorletzter = anf;
                while (vorletzter.next != null && vorletzter.next.name.CompareTo(DelName) != 0)
                    vorletzter = vorletzter.next;

                if (vorletzter.next != null)    // Gesuchtes Element gefunden!
                {
                    anz--;
                    vorletzter.next = vorletzter.next.next;
                    // d.h. nächstes Element wird übersprungen, d.h. gelöscht
                }
                if (vorletzter.next == null)        // Fall 4
                    ende = vorletzter;
            }
        }
        #endregion
    }
}
