﻿using System;
using System.Collections.Generic;
using System.Text;

namespace _04_2_DoppeltVerketteteListe
{
    // Kriterien einfach vs. doppel-verkettete Listen
    // - Speicherplatz
    // - Welche Operationen werden wie HÄUFIG benötigt (Anfügen am Ende, Anfügen am Anf)
    //          Löschen am Anfang --> einfach-verkettet
    // - Löschen des letzten Elements HÄUFIG benötigt? --> Doppelt-verk.L
    class Personenliste
    {
        class LItem
        {
            public string name;
            public LItem next = null, prev = null;
            public LItem(string Name) { name = Name; }

            public override string ToString()
            => $"Name: {name}, prev: {(prev == null ? "null" : prev.name)}, next: {(next == null ? "null" : next.name)}";
        }

        LItem first = null, last = null;

        public void AddEnd(string Name)
        {
            LItem newItem = new LItem(Name);
            if (first == null)
                first = last = newItem;
            else  // Es gibt schon Listenelemente
            {
                last.next = newItem;
                newItem.prev = last;
                last = newItem;
            }
        }
        public void AddFront(string Name)
        {
            LItem newItem = new LItem(Name);
            if (first == null)
                first = last = newItem;
            else
            {
                newItem.next = first;
                first.prev = newItem;
                first = newItem;
            }
        }
        public void AddSorted(string InsName)
        {
            // 1. Fall: Leere Liste oder Anfügen am Listenende
            if (first == null || last.name.CompareTo(InsName) <= 0)
                AddEnd(InsName);
            else if (InsName.CompareTo(first.name) <= 0) 
                // 2. Fall: Anfügen am Listenende
                AddFront(InsName);
            else
            {
                // Wir wissen: Das neue Element ist NICHT das erste und nicht das letzte Element
                LItem newItem = new LItem(InsName);

                LItem item = first;
                while (item.next.name.CompareTo(InsName) < 0)
                    item = item.next;
                // bis hierher identisch zur einfach-verketteten Liste
                newItem.next = item.next; // Vorwärtsverkettung vom neuen Element zur Restliste
                newItem.prev = item;      // Rückwärtsverkettung vom neuen Elenent zum Listenanfangsstück
                item.next.prev = newItem; // Rückwärtsverkettung vom nachfolgenden Listenelement
                item.next = newItem;      // Vorwärtsverkettung zum neuen Element
            }
        }
        public void DeleteFirst()
        {
            // Fall 1: Liste ist leer
            // Fall 2: Liste besteht nur aus EINEM Element
            // Fall 3: Liste hat mehr als ein Element

            if (first == null)  // Fall 1
                return;
            if (first == last)  // Fall 2
                first = last = null;
            else
            {
                first = first.next;
                first.prev = null;
            }
        }
        public void DeleteLast()
        {
            // Fall 1: Liste ist leer
            // Fall 2: Liste besteht nur aus EINEM Element
            // Fall 3: Liste hat mehr als ein Element

            if (first == null)  // Fall 1
                return;
            if (first == last)  // Fall 2
                first = last = null;
            else
            {
                last = last.prev;
                last.next = null;
            }
        }
        public void DeleteByName(string DelName)
        {
            // Fall 1: Liste ist leer
            // Fall 2: Das gesuchte Element befindet sich am Anfang
            // Fall 3: Das gesuchte Element befindet sich am Ende
            // Fall 4: Das gesuchte Element befindet sich mittendrin

            if (first == null)  // Fall 1
                return;
            if (first.name.CompareTo(DelName) == 0)   // Fall 2
                DeleteFirst();
            else if (last.name.CompareTo(DelName) == 0) // Fall 3
                DeleteLast();
            else
            {
                // Fall 4: Wir wissen:
                // Das zu löschende Element befindet sich weder am Anfang noch am Ende
                //    --> Es gibt ein Vorgänger- UND ein Nachfolger-Element
                LItem item = first.next;
                while (item.next != null && item.name.CompareTo(DelName) != 0)
                    item = item.next;
                if (item.next != null) 
                {
                    // d.h. Element wurde gefunden und item zeigt auf dieses Element
                    //                                  20 - 21 - 22
                    item.prev.next = item.next;      // 20 -> 22 Nachfolger
                    item.next.prev = item.prev;      // 22 -> 20 Vorgänger
                }
            }
        }
        public void  Print()
        {
            for (LItem item = first;item!=null;item=item.next)
            {
                Console.WriteLine(item.name);
            }
            Console.WriteLine("-------------------");
        }
        public void PrintReverse()
        {
            for (LItem item = last; item != null; item = item.prev)
            {
                Console.WriteLine(item.name);
            }
            Console.WriteLine("-------------------");
        }
    }
}
