﻿using System;
using System.Diagnostics.CodeAnalysis;

namespace _08_2_Generics_Intro
{
    class Person : IComparable<Person>
    {
        string name, vorname;
        public Person(string Name, string Vorname) { name = Name; vorname = Vorname; }

        public int CompareTo(Person other)
        {
            int comp = name.CompareTo(other.name);
            return (comp != 0) ? comp : vorname.CompareTo(other.vorname);
            //string vollname1 = name + vorname;                  // "MeierAnton"
            //string vollname2 = other.name + other.vorname;      // "MeierBerta"
            //return vollname1.CompareTo(vollname2);              // "MeierAnton".CompareTo("MeierBerta")
        }
        public override string ToString() => $"{vorname} {name}";
    }
    class Program
    {
        //static int Min(int a, int b)
        //{
        //    return (a.CompareTo(b) < 0) ? a : b;
        //}
        //static double Min(double a, double b)
        //{
        //    return (a < b) ? a : b;
        //}
        //static int Min(int a, int b)
        //{
        //    return (a < b) ? a : b;
        //}
        //static string Min(string a, string b)
        //{
        //    return (a.CompareTo(b) < 0) ? a : b;
        //}
        static T Min<T>(T a, T b) where T : IComparable<T>
        {       // IComparable<TYP>: int CompareTo(T other)   |  "Nur" IComparable: int CompareTo(object other)
            return (a.CompareTo(b) < 0) ? a : b;   // a.CompareTo(b)  :: a-b
        }




        //static object Min(object a, object b) 
        //{
        //    return (a.CompareTo(b) < 0) ? a : b;
        //}
        // Dieser Aufruf ist nur dann gültig, wenn der Typ von a eine Methode CompareTo() enthält
        // => Es muss sichergestellt sein, dass Typ T eine Methode CompareTo() besitzt

        // Constraint: Alle Typen sind erlaubt, welche die geforderten Eigenschaften (=Methoden) besitzen
        // Hier: IComparable (=CompareTo)

        // where ~ Vertrag zwischen der Methode und der Verwendung
        // Methode sagt, ich kann ..., WENN mir eine oder mehrere geforderte Methoden bereitgestellt werden
        // Verwendung sagt, ich VERPFLICHTE mich, diese Methode bereitzustellen
        // Compiler überwacht die Einhaltung des Vertrags

        //static int Min_int(int a, int b) 
        //{
        //    return (a.CompareTo(b) < 0) ? a : b;
        //}
        //static double Min_double(double a, double b)
        //{
        //    return (a.CompareTo(b) < 0) ? a : b;
        //}

        static void Main(string[] args)
        {
            Console.WriteLine($"Minimum von 4 und 7: {Min(4, 7)}");
            Console.WriteLine($"Minimum von 7.5 und 7.3: {Min(7.5, 7)}");
            Console.WriteLine($"Minimum von Berta und Anton: {Min("Berta", "Anton")}");
            Console.WriteLine($"Minimum von zwei Person-Objekten: {Min(new Person("Meier", "Anton"), new Person("Meier", "Berta"))}");

            bool gleich = new Person("Meier", "Anton").Equals(new Person("Meier", "Berta"));
            // Equals ist bereits in der Klasse Object definiert, damit in allen Klassen verfügbar,
            //   enthält aber nur die "Basis-Implementierung" eines Referenzvergleichs.
            // MUSS für ein richtiges Equals in der jeweiligen Klasse neu definiert werden.
        }
    }
}
