﻿using System;
using System.Collections.Generic;
using System.Text;

namespace _08_3_GenericList
{
    abstract class Kfz : IComparable<Kfz>     // KfzCoolStyle
        // Ist die Basisklasse von mehreren kfz-Ausprägungen
    {
        // public string kennzeichen;  -- Nicht public machen!

        protected   string kennzeichen;
        // protected ~ öffentlich für abgeleitete Klassen
        //           ~ private für alle anderen

        public int CompareTo(Kfz other)
            => kennzeichen.CompareTo(other.kennzeichen);

        public Kfz(string Kennzeichen) { kennzeichen = Kennzeichen; }
        // Initialisierung bzw. Parameterübernahme unbedingt über
        // den Konstruktor realisieren!!!

        // public virtual double Steuern() { return -1; }
        // virtual ~ Diese Methode kann/darf in einer 
        // von Kfz abgeleiteten Klasse überschrieben werden
        // (Dies muss aber nicht gemacht werden!)

        public abstract double Steuern();
        // abstract ~ Diese Methode MUSS in jeder
        // von Kfz abgeleiteten Klasse überschrieben werden

        // Abstrakte Klasse --> von dieser Klasse können keine Objekte angelegt werden
    }

    class Pkw : Kfz   // Pkw ist-ein Kfz
        // Pkw ist eine Spezialisierung eines Kfz
        // Pkw ist ein Kfz mit Zusatzeigenschaften
        // Pkw erbt von Kfz
        // Pkw ist von Kfz abgeleitet 
    {
        int hubraum=1599;
        int co2;
        public Pkw(int Hubraum, int CO2, string Kennzeichen)
            : base(Kennzeichen)
        {
            hubraum = Hubraum;
            co2 = CO2;
        }
        public override double Steuern()
        {
            return (hubraum + 99) / 100 * 2;
        }
        public double CO2Wert() => co2;

        public override string ToString()
        => $"Pkw: {kennzeichen}: {hubraum}";
    }

    class Motorrad : Kfz
    {
        public Motorrad(string Kennzeichen, int Hubraum)
            : base (Kennzeichen)
        { hubraum = Hubraum; }
        int hubraum = 250;
        public override double Steuern()
        {
            return (hubraum + 24) / 25 * 1.84;
        }
        public override string ToString()
        => $"Motorrad: {kennzeichen}: {hubraum}";
    }

    class LKW : Kfz
    {
        int gewicht;
        int hubraum;
        public LKW(string Kennzeichen, int Gewicht, int Hubraum) 
            : base(Kennzeichen)
            // Erst wird der Basisklassenkonstruktor aufgerufen
            // DANN die eigene Initialiserung
        {
            // kennzeichen = Kennzeichen;
            // Kein explizites Initialisieren von Basisklassen-Variablen!!!
            gewicht = Gewicht;
            hubraum = Hubraum;
        }
        public override double Steuern()
        {
            return 556;
        }
        public override string ToString()
        => $"LKW: {kennzeichen}: {hubraum}";
    }
    class Tieflader : LKW
    {
        int zuladung;
        public Tieflader(string Kennzeichen, int Gewicht, int Hubraum, int Zuladung)
            : base(Kennzeichen,Gewicht,Hubraum)
        { zuladung = Zuladung; }
        //public override double Steuern() => 999;
        // Wenn in der abgeleiteten Klasse eine Überschreibung der 
        // Basisklassenmethodendefinition erfolgt, wird die Definition
        // aus der Basisklasse verwendet

        public override double Steuern()
        {
            return base.Steuern();
        }
        public override string ToString()
        => $"Tieflader: {kennzeichen}: Zuladung: {zuladung}";
    }
}
